package meerkat.bulletinboard;

import meerkat.protobuf.BulletinBoardAPI.*;
import meerkat.protobuf.Crypto.*;
import meerkat.util.BulletinBoardMessageComparator;

import java.util.LinkedList;
import java.util.List;

/**
 * Created by Arbel Deutsch Peled on 14-Dec-15.
 *
 * A data structure for holding a complete batch message along with its signature
 */
public class CompleteBatch {

    private BeginBatchMessage beginBatchMessage;
    private List<BatchData> batchDataList;
    private Signature signature;

    public CompleteBatch() {
        batchDataList = new LinkedList<BatchData>();
    }

    public CompleteBatch(BeginBatchMessage newBeginBatchMessage) {
        this();
        beginBatchMessage = newBeginBatchMessage;
    }

    public CompleteBatch(BeginBatchMessage newBeginBatchMessage, List<BatchData> newDataList) {
        this(newBeginBatchMessage);
        appendBatchData(newDataList);
    }

    public CompleteBatch(BeginBatchMessage newBeginBatchMessage, List<BatchData> newDataList, Signature newSignature) {
        this(newBeginBatchMessage, newDataList);
        signature = newSignature;
    }

    public BeginBatchMessage getBeginBatchMessage() {
        return beginBatchMessage;
    }

    public List<BatchData> getBatchDataList() {
        return batchDataList;
    }

    public Signature getSignature() {
        return signature;
    }

    public CloseBatchMessage getCloseBatchMessage() {
        return CloseBatchMessage.newBuilder()
                .setBatchId(getBeginBatchMessage().getBatchId())
                .setBatchLength(getBatchDataList().size())
                .setSig(getSignature())
                .build();
    }

    public void setBeginBatchMessage(BeginBatchMessage beginBatchMessage) {
        this.beginBatchMessage = beginBatchMessage;
    }

    public void appendBatchData(BatchData newBatchData) {
        batchDataList.add(newBatchData);
    }

    public void appendBatchData(List<BatchData> newBatchDataList) {
        batchDataList.addAll(newBatchDataList);
    }

    public void setSignature(Signature newSignature) {
        signature = newSignature;
    }

    @Override
    public boolean equals(Object other) {

        if (!(other instanceof CompleteBatch)) {
            return false;
        }

        CompleteBatch otherBatch = (CompleteBatch) other;

        boolean result = true;

        if (beginBatchMessage == null) {
            if (otherBatch.getBeginBatchMessage() != null)
                return false;
        } else {
            result = result && beginBatchMessage.equals(otherBatch.getBeginBatchMessage());
        }

        if (batchDataList == null) {
            if (otherBatch.getBatchDataList() != null)
                return false;
        } else {
            result = result && batchDataList.equals(otherBatch.getBatchDataList());
        }

        if (signature == null) {
            if (otherBatch.getSignature() != null)
                return false;
        } else {
            result = result && signature.equals(otherBatch.getSignature());
        }

        return result;

    }

    @Override
    public String toString() {
        return "Batch " + beginBatchMessage.getSignerId().toString() + ":" + beginBatchMessage.getBatchId();
    }

}
