package meerkat.util;

import meerkat.protobuf.BulletinBoardAPI.*;

import java.util.LinkedList;
import java.util.List;

/**
 * Created by Arbel Deutsch Peled on 16-Feb-16.
 */
public class BulletinBoardUtils {

    /**
     * Searches the tags in the message for one that begins with given prefix
     * @param message is the message to search
     * @param prefix is the given prefix
     * @return the tag without the prefix, if found, or null if not found
     */
    public static String findTagWithPrefix(BulletinBoardMessage message, String prefix) {

        for (String tag : message.getMsg().getTagList()){
            if (tag.startsWith(prefix)) {
                return tag.substring(prefix.length());
            }
        }

        return null;

    }

    /**
     * Searches the tags in a message for tags that do not contain a given list of prefixes
     * @param message is the message to search
     * @param prefixes is the list of prefixes
     * @return a list of the tags that do *not* contain any of the given prefixes
     */
    public static List<String> removePrefixTags(BulletinBoardMessage message, Iterable<String> prefixes) {

        if (prefixes == null)
            return message.getMsg().getTagList();

        List<String> result = new LinkedList<>();

        for (String tag : message.getMsg().getTagList()){

            boolean found = false;

            for (String prefix : prefixes){
                if (tag.startsWith(prefix)){
                    found = true;
                    break;
                }
            }

            if (!found) {
                result.add(tag);
            }

        }

        return result;

    }

}
